function wait(frames)
  stop_frame = emu.framecount() + frames
  repeat
    emu.frameadvance()
  until (emu.framecount() >= stop_frame)
end

function round(x)
  return math.floor(x + .5)
end

function hi(x)
  return ( AND(x,0xFF00) / 256 )
end

function lo(x)
  return AND(x,0xFF)
end

function abscents(x,y)
  return ( math.abs( 1200 * (math.log(x/y)/math.log(2)) ) )
end

function key(x)
  return ( (math.log(x/440)/math.log(2)) * 12 + 69 )
end

poke = memory.writebyte

MAX_CENTS = 5

NOTE_SECONDS = .5
REST_SECONDS = .25

MIN_P_TRI = 2
MIN_P_PULSE = 8
MAX_P = 2047

TAB = "	"

if(type(machine) == "nil") then
  machine = "NTSC"
end

if (machine == "NTSC") then

  cpu_frequency = 19687500 / 11
  noise_period_table = { [0]=4, 8, 16, 32, 64, 96, 128, 160, 202, 254, 380, 508, 762, 1016, 2034, 4068 }
  dpcm_period_table = { [0]=428, 380, 340, 320, 286, 254, 226, 214, 190, 160, 142, 128, 106, 84, 72, 54 }
  NOTE_FRAMES = round( 60 * NOTE_SECONDS )
  REST_FRAMES = round( 60 * REST_SECONDS )

elseif (machine == "PAL") then

  -- cpu_frequency = 53203425 / 32  -- from nesdev
  cpu_frequency    = 26601714 / 16  -- from FECUX source
  noise_period_table = { [0]=4, 7, 14, 30, 60, 88, 118, 148, 188, 236, 354, 472, 708, 944, 1890, 3778 }
  dpcm_period_table = { [0]=398, 354, 316, 298, 276, 236, 210, 198, 176, 148, 132, 118, 98, 78, 66, 50 }
  NOTE_FRAMES = round( 50 * NOTE_SECONDS )
  REST_FRAMES = round( 50 * REST_SECONDS )

end


  -- Set up sound channels

poke(0x400C,0x3F) -- noise: manual duration, manual envelope, max volume
poke(0x400E,0x80) -- noise: short mode
poke(0x4000,0xBF) -- pulse 1: 50% duty, manual duration, manual envelope, max volume
poke(0x4001,8)    -- pulse 1: sweep disabled, sweep negate (to allow low notes)
poke(0x4008,0xFF) -- triangle: manual duration, non-zero linear counter


emu.print(machine)

emu.print("srt noi" ..TAB            ..TAB           ..TAB             ..TAB             ..TAB..
          "pulse"   ..TAB            ..TAB           ..TAB             ..TAB             ..TAB..
          "tri"                                                                                  )

emu.print("raw"     ..TAB.. "calc f" ..TAB.. "get f" ..TAB.. "calckey" ..TAB.. "get key" ..TAB..
          "raw"     ..TAB.. "calc f" ..TAB.. "get f" ..TAB.. "calckey" ..TAB.. "get key" ..TAB..
          "raw"     ..TAB.. "calc f" ..TAB.. "get f" ..TAB.. "calckey" ..TAB.. "get key"         )


for n = 0,15 do

  fn = cpu_frequency / ( noise_period_table[n] * 93 )
  pp = round( ( cpu_frequency / (16 * fn) ) - 1 )
  fp = cpu_frequency / ( (pp + 1) * 16 )
  pt = round( ( cpu_frequency / (32 * fn) ) - 1 )
  ft = cpu_frequency / ( (pt + 1) * 32 )

  do_pulse = ( (pp >= MIN_P_PULSE) and (pp <= MAX_P) and (abscents(fn,fp) < MAX_CENTS) )
  do_tri   = ( (pt >= MIN_P_TRI  ) and (pt <= MAX_P) and (abscents(fn,ft) < MAX_CENTS) )

  poke(0x400E,0x80 + n) -- noise: short mode + noise period

  if(do_pulse) then
    poke(0x4002,lo(pp)) -- pulse 1: period low
    poke(0x4003,hi(pp)) -- pulse 1: period high
  end

  if(do_tri) then
    poke(0x400A,lo(pt)) -- triangle: period low
    poke(0x400B,hi(pt)) -- triangle: period high
  end

  snd = sound.get().rp2a03

  kn = key(fn)
  gfn = snd.noise.frequency
  gkn = snd.noise.midikey

  if(do_pulse) then
    kp = key(fp)
    gfp= snd.square1.frequency
    gkp= snd.square1.midikey
  else
    pp="--"
    fp="--"
    kp ="--"
    gfp="--"
    gkp="--"
  end

  if(do_tri) then
    kt = key(ft)
    gft= snd.triangle.frequency
    gkt= snd.triangle.midikey
  else
    pt="--"
    ft="--"
    kt ="--"
    gft="--"
    gkt="--"
  end

  emu.print(n  ..TAB.. fn ..TAB.. gfn ..TAB.. kn ..TAB.. gkn ..TAB..
            pp ..TAB.. fp ..TAB.. gfp ..TAB.. kp ..TAB.. gkp ..TAB..
            pt ..TAB.. ft ..TAB.. gft ..TAB.. kt ..TAB.. gkt         )

  poke(0x4015,8)                -- enable noise channel
  poke(0x400F,8)                -- start noise
  wait(NOTE_FRAMES)
  poke(0x4015,0)                -- stop/disable all channels
  wait(REST_FRAMES)

  if(do_pulse) then
    poke(0x4015,1)              -- enable pulse 1 channel    
    poke(0x4003,0xF8 + hi(pp))  -- start pulse 1 + pulse 1 period high
    wait(NOTE_FRAMES)
    poke(0x4015,0)              -- stop/disable all channels
    wait(REST_FRAMES)
  end

  if(do_tri) then
    poke(0x4015,4)              -- enable triangle channel    
    poke(0x400B,0xF8 + hi(pt))  -- start triangle + triangle period high
    wait(NOTE_FRAMES)
    poke(0x4015,0)              -- stop/disable all channels
    wait(REST_FRAMES)
  end

  wait(NOTE_FRAMES + REST_FRAMES)

end

